import { CacheModule, Module } from '@nestjs/common';
import { SlonikModule } from 'nestjs-slonik';
import { RedisClientOptions } from 'redis';
import { postgresConnectionUrl } from './config/database.config';
import { redisConfig } from './config/redis.config';
import { UserModule } from './modules/user/user.module';
import { APP_INTERCEPTOR } from '@nestjs/core';
import { ContextInterceptor } from './libs/application/context/ContextInterceptor';
import { ExceptionInterceptor } from './libs/application/interceptors/exception.interceptors';
import { RequestContextModule } from 'nestjs-request-context';
import { CqrsModule } from '@nestjs/cqrs';
import { EventEmitterModule } from '@nestjs/event-emitter';

const interceptors = [
  {
    provide: APP_INTERCEPTOR,
    useClass: ContextInterceptor
  },
  {
    provide: APP_INTERCEPTOR,
    useClass: ExceptionInterceptor
  }
];

@Module({
  imports: [
    EventEmitterModule.forRoot(),
    RequestContextModule,
    SlonikModule.forRoot({
      connectionUri: postgresConnectionUrl,
    }),
    CqrsModule,
    CacheModule.register<RedisClientOptions>(redisConfig),
    UserModule
  ],
  controllers: [],
  providers: [...interceptors],
})
export class AppModule {}
