import { Post, ConflictException as ConflictHttpException, Body, Controller } from "@nestjs/common";
import { match, Result} from 'oxide.ts';
import { routesV1 } from "@src/config/app.routes";
import { UserAlreadyExistsError } from "../domain/user.error";
import { UserResponseDto } from "../dtos/user.response.dto";
import { CreateUserCommand } from "./create-user.command";
import { CommandBus } from "@nestjs/cqrs";
import { CreateUserRequestDto } from "./create-user.request.dto";

@Controller(routesV1.version)
export class CreateUserHttpController {
  constructor(private readonly commandBus: CommandBus) {}

  @Post(routesV1.user.root)
  async create(@Body() body: CreateUserRequestDto): Promise<UserResponseDto> {
    const command = new CreateUserCommand(body);

    const result: Result<UserResponseDto, UserAlreadyExistsError> = 
      await this.commandBus.execute(command);
    
    return match(result, {
      Ok: (res: any) => res,
      Err: (error: Error) => {
        if(error instanceof UserAlreadyExistsError)
          throw new ConflictHttpException(error.message);
        throw error;
      }
    })
  }
}