import { Injectable } from "@nestjs/common";
import { Mapper } from "@src/libs/ddd";
import { UserModel, userSchema } from "./database/user.repository";
import { UserEntity } from "./domain/user.entity";
import { UserResponseDto } from "./dtos/user.response.dto";

@Injectable()
export class UserMapper implements Mapper<UserEntity, UserModel, UserResponseDto> {
  toPersistence(entity: UserEntity): UserModel {
    const copy = entity.getPropsCopy();

    const record: UserModel = {
      id: copy.id,
      email: copy.email,
      phone_number: copy.phone_number,
      password: copy.password,
      role_id: copy.role_id,
      fullname: copy.fullname,
      created_at: copy.createdAt,
      updated_at: copy.updatedAt,
    }

    return userSchema.parse(record)
  }

  toDomain(record: UserModel): UserEntity {
    const entity = new UserEntity({
      id: record.id,
      props: {
        email: record.email,
        fullname: record.fullname,
        password: record.password,
        phone_number: record.phone_number,
        role_id: record.role_id
      }
    });

    return entity;
  }

  toResponse(entity: UserEntity): UserResponseDto {
    const props = entity.getPropsCopy();
    const response = new UserResponseDto(entity);
    response.email = props.email;
    response.fullname = props.fullname;
    response.phone_number = props.phone_number;
    return response;
  }

}