import { Logger, Module, Provider } from '@nestjs/common';
import { CqrsModule } from '@nestjs/cqrs';
import { CreateUserHttpController } from './commands/create-user.http.controller';
import { CreateUserMessageController } from './commands/create-user.message.controller';
import { CreateUserService } from './commands/create-user.service';
import { UserRepository } from './database/user.repository';
import { USER_REPOSITORY } from './user.di-tokens';
import { UserMapper } from './user.mapper';


const httpControllers = [
  CreateUserHttpController
];

const messageControllers = [CreateUserMessageController];
const commandHandlers: Provider[] = [CreateUserService];
const mappers: Provider[] = [UserMapper];
const repositories: Provider[] = [
  { provide: USER_REPOSITORY, useClass: UserRepository}
];

@Module({
  imports: [CqrsModule],
  controllers: [...httpControllers, ...messageControllers],
  providers: [
    Logger,
    ...repositories,
    ...mappers,
    ...commandHandlers
  ],
})
export class UserModule {}
